@@ -124,7 +124,7 @@ transfer <- function(nsteps=500,dmode='coupled',devmax=0.1,
           myib <- c(myib,ib)
           ibasis <- ibuf[i]
           buffargs <- list(logK=logK,ibasis=ibasis,logact.basis=logact.basis,is.buffer=ib)
-          bresult <- do.call(buffer,buffargs)$logact.basis[ibuf[i]]
+          bresult <- do.call("buffer",buffargs)$logact.basis[ibuf[i]]
           if(i==1) br <- bresult else br <- c(br,bresult)
         }
         bresult <- br
@@ -144,7 +144,7 @@ transfer <- function(nsteps=500,dmode='coupled',devmax=0.1,
           ibasis <- ibuf[i]
           buffargs <- list(logK=buffargs$logK,ibasis=ibasis,
             logact.basis=buffargs$logact.basis,is.buffer=ib)
-          bresult <- do.call(buffer,buffargs)$logact.basis[ibuf[i]]
+          bresult <- do.call("buffer",buffargs)$logact.basis[ibuf[i]]
           if(i==1) br <- bresult else br <- c(br,bresult)
         }
         bresult <- br
@@ -372,7 +372,7 @@ transfer <- function(nsteps=500,dmode='coupled',devmax=0.1,
       # chemical affinities (near-equilibrium rates)
       # 20090409 use abundance here -- relative
       # abundances of species in equilibrium
-      molspecies3 <- 10^as.numeric(abundance(myaff,rep(1,length(myaff)),0))
+      molspecies3 <- 10^as.numeric(abundance.new(myaff,rep(1,length(myaff)),0))
     }
     # the number of moles of basis species used
     ipresent <- which(log10(molspecies3) > logpresent)
